package api

import (
	"bytes"
	"encoding/json"
	"errors"
	"io/ioutil"
	"net/http"
	"strconv"

	"github.com/golang-jwt/jwt"
)

// Method must be a valid REST method, like GET, POST, PUT, DELETE etc
func (api Api) Call(method string, path string, payload []byte) (ApiRes, error) {
	respObj := ApiRes{}

	adminToken, _, err := api.getAdminToken()
	if err != nil {
		return ApiRes{}, err
	}

	req, err := http.NewRequest(method, api.URL+path, bytes.NewBuffer(payload))
	if err != nil {
		api.Log.Error("Could not create request", "method", method, "path", path, "err", err.Error())
		return ApiRes{}, err
	}
	req.Header.Set("Content-Type", "application/json")
	req.Header.Set("Authorization", "bearer "+adminToken)

	httpClient := &http.Client{}
	res, err := httpClient.Do(req)
	if err != nil {
		api.Log.Error("Could not call backend API", "method", method, "path", path, "err", err.Error())
		return ApiRes{}, err
	}
	defer res.Body.Close()

	api.Log.Debug("API call res status", "method", method, "path", path, "status", res.StatusCode)
	respObj.StatusCode = res.StatusCode

	if strconv.Itoa(res.StatusCode)[0:1] == "5" {
		api.Log.Error("API gave internal server error", "method", method, "path", path, "statusCode", res.StatusCode)
		return ApiRes{}, errors.New("API gave internal server error")
	}

	body, err := ioutil.ReadAll(res.Body)
	if err != nil {
		api.Log.Error("Could not read res body", "method", method, "path", path, "err", err.Error())
		return ApiRes{}, err
	}

	respObj.Body = body

	return respObj, nil
}

func (api Api) getAdminToken() (string, AdminClaims, error) {
	if api.AdminToken == "" {
		// No adminToken exists, obtain it

		adminToken, err := api.getAdminTokenCall()
		api.AdminToken = adminToken

		if err != nil {
			api.Log.Error("Could not get admin token")
			return "", AdminClaims{}, err
		}

		parsedToken, err := api.parseAdminToken(api.AdminToken)
		if err != nil {
			api.Log.Error("Could not parse admin token")
			return "", AdminClaims{}, err
		}

		return api.AdminToken, parsedToken, err
	}

	parsedToken, err := api.parseAdminToken(api.AdminToken)
	if err != nil {
		api.Log.Error("Could not parse admin token")
		return "", AdminClaims{}, err
	}

	return api.AdminToken, parsedToken, nil
}

func (api Api) getAdminTokenCall() (string, error) {
	req, err := http.NewRequest("POST", api.URL+"/auth/api-key", bytes.NewBufferString("\""+api.AdminApiKey+"\""))
	if err != nil {
		api.Log.Error("Could not create request", "err", err.Error())
		return "", err
	}
	req.Header.Set("Content-Type", "application/json")

	httpClient := &http.Client{}
	res, err := httpClient.Do(req)
	if err != nil {
		api.Log.Error("Could not call backend API", "err", err.Error())
		return "", err
	}
	defer res.Body.Close()

	api.Log.Debug(api.URL+"/auth/api-key res status", "status", res.StatusCode)

	if strconv.Itoa(res.StatusCode)[0:1] == "5" {
		api.Log.Error("API gave internal server error", "statusCode", res.StatusCode)
		return "", errors.New("API gave internal server error")
	} else if res.StatusCode != 200 {
		api.Log.Error("API gave unexpected statusCode", "statusCode", res.StatusCode)
		return "", errors.New("API gave unexpected statusCode")
	}

	body, err := ioutil.ReadAll(res.Body)
	if err != nil {
		api.Log.Error("Could not read res body", "err", err.Error())
		return "", err
	}

	var jsonResp TokenRes

	err = json.Unmarshal(body, &jsonResp)
	if err != nil {
		api.Log.Error("Could not parse JSON from API", "err", err.Error())
		return "", err
	}

	return jsonResp.Jwt, nil
}

func (api Api) parseAdminToken(token string) (AdminClaims, error) {
	claims := &AdminClaims{}
	parsedToken, err := jwt.ParseWithClaims(token, claims, func(token *jwt.Token) (interface{}, error) {
		return []byte(api.JwtSharedSecret), nil
	})

	if err != nil {
		api.Log.Error("Could not parse admin token", "err", err.Error())
		return AdminClaims{}, err
	}

	if !parsedToken.Valid {
		err := errors.New("invalid token")
		api.Log.Error("Invalid token")
		return AdminClaims{}, err
	}

	claims, ok := parsedToken.Claims.(*AdminClaims)
	if !ok {
		err := errors.New("parsedToken.Claims.(*AdminClaims) did not return ok")
		api.Log.Error(err.Error())
		return AdminClaims{}, err
	}

	return *claims, nil
}
