package handlers

import (
	"encoding/json"
	"strconv"

	"github.com/gofiber/fiber/v2"

	"gitlab.larvit.se/power-plan/auth-ui/src/api"
	"gitlab.larvit.se/power-plan/auth-ui/src/utils"
	"gitlab.larvit.se/power-plan/auth-ui/src/views"
)

type LoginUser struct {
	Username string `json:"name"`
	Password string `json:"password"`
}

// Index GET
func (h Handlers) Index(c *fiber.Ctx) error {
	c.Set(fiber.HeaderContentType, fiber.MIMETextHTML)

	user, userLoggedIn := c.Locals("user").(utils.AuthClaims)
	if !userLoggedIn {
		user = utils.AuthClaims{}
	}

	userData := views.UserData{
		Name: user.AccountName,
	}

	return c.SendString(views.Index(views.IndexData{
		UserData:     userData,
		UserLoggedIn: userLoggedIn,
	}))
}

// Index POST
func (h Handlers) IndexPost(c *fiber.Ctx) error {
	c.Set(fiber.HeaderContentType, fiber.MIMETextHTML)
	loginUser := new(LoginUser)

	err := c.BodyParser(loginUser)
	if err != nil {
		h.Log.Debug("Invalid input data", "err", err.Error())
		c.Status(400)
	}

	jsonPayload, err := json.Marshal(loginUser)
	if err != nil {
		h.Log.Error("Could not marshal loginUser struct", "err", err.Error())
		return c.Status(500).SendString(views.Error500())
	}

	apiRes, err := h.Api.Call("POST", "/auth/password", jsonPayload)
	if err != nil {
		h.Log.Error("Could not call backend API", "err", err.Error())
		return c.Status(500).SendString(views.Error500())
	}

	if apiRes.StatusCode == 200 {
		err = utils.SetTokenCookies(apiRes.Body, c, h.Log)
		if err != nil {
			h.Log.Error("Could not unmarshal body from auth API", "err", err.Error())
			return c.Status(500).SendString(views.Error500())
		}

		tokens, err := utils.ParseTokenApiResBody(apiRes.Body, h.Log)
		if err != nil {
			return c.Status(500).SendString(views.Error500())
		}

		parsed, err := utils.ParseAndValidateToken(tokens.Jwt, h.Api.JwtSharedSecret, h.Log)
		if err != nil {
			h.Log.Error("Could not parse and validate new user token after renewal", "err", err.Error())
			h.Log.Debug("Broken token", "tokens.Jwt", tokens.Jwt, "apiRes.Body", string(apiRes.Body))
			return c.Status(500).SendString(views.Error500())
		}

		c.Locals("user", parsed)

		return c.SendString(views.Index(views.IndexData{
			OkMsg:        "User logged in!",
			Title:        "Dashboard",
			UserLoggedIn: true,
		}))
	} else if strconv.Itoa(apiRes.StatusCode)[0:1] == "4" {
		var jsonResp []api.ApiResError

		err := json.Unmarshal(apiRes.Body, &jsonResp)
		if err != nil {
			h.Log.Error("Could not unmarshal api error response", "err", err.Error())
			return c.Status(500).SendString(views.Error500())
		}

		return c.Status(apiRes.StatusCode).SendString(views.Index(views.IndexData{
			ErrField: jsonResp[0].Field,
			ErrMsg:   jsonResp[0].Error,
			Title:    "Login",
		}))
	} else {
		h.Log.Error("Unexpected API response status", "status", apiRes.StatusCode)
		return c.Status(500).SendString(views.Error500())
	}
}
