package handlers

import (
	"errors"
	"strings"

	"github.com/gofiber/fiber/v2"
	"gitlab.larvit.se/power-plan/auth-ui/src/utils"
	"gitlab.larvit.se/power-plan/auth-ui/src/views"
)

// Log all requests.
func (h Handlers) LogReq(c *fiber.Ctx) error {
	h.Log.Debug("http request", "method", c.Method(), "url", c.OriginalURL())

	c.Next()
	return nil
}

// Token cookie handling.
func (h Handlers) HandleCookieTokens(c *fiber.Ctx) error {
	userToken := c.Cookies("userToken")

	// If no userToken exists, nothing more to do, so exiting.
	if len(userToken) == 0 {
		return c.Next()
	}

	parsed, err := utils.ParseAndValidateToken(userToken, h.Api.JwtSharedSecret, h.Log)
	// If parsing went fine, just set the user context local and exit.
	if err == nil {
		c.Locals("user", parsed)
		return c.Next()
	}

	// Something is wrong and it is not that the token has expired.
	// Log the error and exit.
	if !strings.HasPrefix(err.Error(), "token is expired") {
		h.Log.Error("could not parse token with claims", "err", err.Error())
		return c.Next()
	}

	renewalToken := c.Cookies("renewalToken")

	if len(renewalToken) == 0 {
		h.Log.Debug("no renewalToken cookie set, can not renew expired token")
		return errors.New("no renewalToken cookie set, can not renew expired token")
	}

	apiRes, err := h.Api.Call("POST", "/renew-token", []byte("\""+renewalToken+"\""))
	if err != nil {
		h.Log.Error("Could not call backend API", "err", err.Error())
		return c.Status(500).SendString(views.Error500())
	}

	if apiRes.StatusCode != 200 {
		h.Log.Error("Unexpected API response status", "status", apiRes.StatusCode)
		return c.Status(500).SendString(views.Error500())
	}

	h.Log.Debug("Setting new token cookies from", "apiRes.Body", apiRes.Body)

	err = utils.SetTokenCookies(apiRes.Body, c, h.Log)
	if err != nil {
		h.Log.Error("Could not unmarshal body from auth API", "err", err.Error())
		return c.Status(500).SendString(views.Error500())
	}

	userToken = c.Cookies("userToken")
	parsed, err = utils.ParseAndValidateToken(userToken, h.Api.JwtSharedSecret, h.Log)
	if err != nil {
		h.Log.Error("Could not parse and validate new user token after renewal", "err", err.Error())
		return c.Status(500).SendString(views.Error500())
	}

	c.Locals("user", parsed)
	return c.Next()
}
