package utils

import (
	"encoding/json"
	"fmt"
	"log"
	"strings"
	"time"

	"github.com/gofiber/fiber/v2"
	"github.com/golang-jwt/jwt"
	"go.uber.org/zap"
	"go.uber.org/zap/zapcore"
)

func SyslogTimeEncoder(t time.Time, enc zapcore.PrimitiveArrayEncoder) {
	loc, _ := time.LoadLocation("UTC")
	t = t.In(loc)

	enc.AppendString(t.Format("2006-01-02 15:04:05"))
}

func GetLog() *zap.SugaredLogger {
	cfg := zap.NewProductionConfig()
	cfg.Development = true
	cfg.DisableCaller = false
	cfg.DisableStacktrace = false
	cfg.Encoding = "console" // "console" or "json"
	cfg.EncoderConfig.EncodeTime = SyslogTimeEncoder
	cfg.OutputPaths = []string{"stdout"}
	cfg.ErrorOutputPaths = []string{"stderr"}
	cfg.Level.SetLevel(zap.DebugLevel)

	logger, err := cfg.Build()
	if err != nil {
		log.Panicf("Could not build logger, err: %v", err)
	}
	defer logger.Sync() // Flushes buffer, if any
	log := logger.Sugar()

	return log
}

type AuthClaims struct {
	jwt.StandardClaims
	AccountId     string              `json:"accountId"`
	AccountFields map[string][]string `json:"accountFields"`
	AccountName   string              `json:"accountName"`
}

func ParseAndValidateToken(token string, jwtSecret string, log *zap.SugaredLogger) (AuthClaims, error) {
	var claims AuthClaims

	_, err := jwt.ParseWithClaims(token, &claims, func(token *jwt.Token) (interface{}, error) {
		_, ok := token.Method.(*jwt.SigningMethodHMAC)
		if !ok {
			log.Error("unexpected signing method", "signingMethod", token.Header["alg"])
			return nil, fmt.Errorf("unexpected signing method: %v", token.Header["alg"])
		}
		return []byte(jwtSecret), nil
	})
	if err != nil {
		if strings.HasPrefix(err.Error(), "token is expired") {
			log.Debug("token have expired", "err", err.Error())
		} else {
			log.Error("could not parse token with claims", "err", err.Error())
		}
		return AuthClaims{}, err
	}

	return claims, nil
}

type AuthRes struct {
	Jwt          string `json:"jwt"`
	RenewalToken string `json:"renewalToken"`
}

func ParseTokenApiResBody(apiResBody []byte, log *zap.SugaredLogger) (AuthRes, error) {
	var authRes AuthRes
	err := json.Unmarshal(apiResBody, &authRes)
	if err != nil {
		log.Error("Could not unmarshal body from auth API", "err", err.Error())
		return AuthRes{}, err
	}

	return authRes, nil
}

func SetTokenCookies(apiResBody []byte, c *fiber.Ctx, log *zap.SugaredLogger) error {
	authRes, err := ParseTokenApiResBody(apiResBody, log)
	if err != nil {
		return err
	}

	log.Debug("setting cookies", "userToken", authRes.Jwt, "len(renewalToken)", len(authRes.RenewalToken))

	c.Cookie(&fiber.Cookie{
		Name:     "userToken",
		Value:    authRes.Jwt,
		Expires:  time.Now().Add(24 * time.Hour),
		HTTPOnly: false,
		SameSite: "lax",
	})

	c.Cookie(&fiber.Cookie{
		Name:     "renewalToken",
		Value:    authRes.RenewalToken,
		Expires:  time.Now().Add(24 * time.Hour),
		HTTPOnly: true,
		SameSite: "lax",
	})

	return nil
}
